@@ -5,30 +5,40 @@ import warnings
 import re
 import inspect
 import uuid
+import collections
 
 from mako.template import Template
 from sqlalchemy.engine import url
 from sqlalchemy import __version__
 
-from .compat import callable, exec_, load_module_py, load_module_pyc, binary_type
+from .compat import callable, exec_, load_module_py, load_module_pyc, \
+    binary_type, string_types, py27
+
 
 class CommandError(Exception):
     pass
 
+
 def _safe_int(value):
     try:
         return int(value)
     except:
         return value
-_vers = tuple([_safe_int(x) for x in re.findall(r'(\d+|[abc]\d)', __version__)])
+_vers = tuple(
+    [_safe_int(x) for x in re.findall(r'(\d+|[abc]\d)', __version__)])
 sqla_07 = _vers > (0, 7, 2)
-sqla_08 = _vers >= (0, 8, 0, 'b2')
+sqla_08 = _vers >= (0, 8, 0)
+sqla_083 = _vers >= (0, 8, 3)
+sqla_084 = _vers >= (0, 8, 4)
 sqla_09 = _vers >= (0, 9, 0)
 sqla_092 = _vers >= (0, 9, 2)
 sqla_094 = _vers >= (0, 9, 4)
+sqla_094 = _vers >= (0, 9, 4)
+sqla_099 = _vers >= (0, 9, 9)
+sqla_100 = _vers >= (1, 0, 0)
 if not sqla_07:
     raise CommandError(
-            "SQLAlchemy 0.7.3 or greater is required. ")
+        "SQLAlchemy 0.7.3 or greater is required. ")
 
 from sqlalchemy.util import format_argspec_plus, update_wrapper
 from sqlalchemy.util.compat import inspect_getfullargspec
@@ -36,12 +46,17 @@ from sqlalchemy.util.compat import inspect_getfullargspec
 import logging
 log = logging.getLogger(__name__)
 
+if py27:
+    # disable "no handler found" errors
+    logging.getLogger('alembic').addHandler(logging.NullHandler())
+
+
 try:
     import fcntl
     import termios
     import struct
     ioctl = fcntl.ioctl(0, termios.TIOCGWINSZ,
-                           struct.pack('HHHH', 0, 0, 0, 0))
+                        struct.pack('HHHH', 0, 0, 0, 0))
     _h, TERMWIDTH, _hp, _wp = struct.unpack('HHHH', ioctl)
     if TERMWIDTH <= 0:  # can occur if running in emacs pseudo-tty
         TERMWIDTH = None
@@ -49,12 +64,14 @@ except (ImportError, IOError):
     TERMWIDTH = None
 
 
-def template_to_file(template_file, dest, **kw):
-    with open(dest, 'w') as f:
+def template_to_file(template_file, dest, output_encoding, **kw):
+    with open(dest, 'wb') as f:
+        template = Template(filename=template_file)
         f.write(
-            Template(filename=template_file).render(**kw)
+            template.render_unicode(**kw).encode(output_encoding)
         )
 
+
 def create_module_class_proxy(cls, globals_, locals_):
     """Create module level proxy functions for the
     methods on a given class.
@@ -97,18 +114,18 @@ def create_module_class_proxy(cls, globals_, locals_):
             defaulted_vals = ()
 
         apply_kw = inspect.formatargspec(
-                                name_args, spec[1], spec[2],
-                                defaulted_vals,
-                                formatvalue=lambda x: '=' + x)
+            name_args, spec[1], spec[2],
+            defaulted_vals,
+            formatvalue=lambda x: '=' + x)
 
         def _name_error(name):
             raise NameError(
-                    "Can't invoke function '%s', as the proxy object has "\
-                    "not yet been "
-                    "established for the Alembic '%s' class.  "
-                    "Try placing this code inside a callable." % (
-                        name, cls.__name__
-                    ))
+                "Can't invoke function '%s', as the proxy object has "
+                "not yet been "
+                "established for the Alembic '%s' class.  "
+                "Try placing this code inside a callable." % (
+                    name, cls.__name__
+                ))
         globals_['_name_error'] = _name_error
 
         func_text = textwrap.dedent("""\
@@ -137,6 +154,7 @@ def create_module_class_proxy(cls, globals_, locals_):
             else:
                 attr_names.add(methname)
 
+
 def write_outstream(stream, *text):
     encoding = getattr(stream, 'encoding', 'ascii') or 'ascii'
     for t in text:
@@ -151,8 +169,10 @@ def write_outstream(stream, *text):
             # as the exception is "ignored" (noisily) in TextIOWrapper.
             break
 
+
 def coerce_resource_to_filename(fname):
-    """Interpret a filename as either a filesystem location or as a package resource.
+    """Interpret a filename as either a filesystem location or as a package
+    resource.
 
     Names that are non absolute paths and contain a colon
     are interpreted as resources and coerced to a file location.
@@ -163,6 +183,7 @@ def coerce_resource_to_filename(fname):
         fname = pkg_resources.resource_filename(*fname.split(':'))
     return fname
 
+
 def status(_statmsg, fn, *arg, **kw):
     msg(_statmsg + " ...", False)
     try:
@@ -173,24 +194,29 @@ def status(_statmsg, fn, *arg, **kw):
         write_outstream(sys.stdout, " FAILED\n")
         raise
 
+
 def err(message):
     log.error(message)
     msg("FAILED: %s" % message)
     sys.exit(-1)
 
+
 def obfuscate_url_pw(u):
     u = url.make_url(u)
     if u.password:
         u.password = 'XXXXX'
     return str(u)
 
+
 def asbool(value):
     return value is not None and \
         value.lower() == 'true'
 
+
 def warn(msg):
     warnings.warn(msg)
 
+
 def msg(msg, newline=True):
     if TERMWIDTH is None:
         write_outstream(sys.stdout, msg)
@@ -204,6 +230,7 @@ def msg(msg, newline=True):
                 write_outstream(sys.stdout, "  ", line, "\n")
         write_outstream(sys.stdout, "  ", lines[-1], ("\n" if newline else ""))
 
+
 def load_python_file(dir_, filename):
     """Load a file from the given path as a Python module."""
 
@@ -215,7 +242,8 @@ def load_python_file(dir_, filename):
             module = load_module_py(module_id, path)
         elif os.path.exists(simple_pyc_file_from_path(path)):
             # look for sourceless load
-            module = load_module_pyc(module_id, simple_pyc_file_from_path(path))
+            module = load_module_pyc(
+                module_id, simple_pyc_file_from_path(path))
         else:
             raise ImportError("Can't find Python file %s" % path)
     elif ext in (".pyc", ".pyo"):
@@ -223,6 +251,7 @@ def load_python_file(dir_, filename):
     del sys.modules[module_id]
     return module
 
+
 def simple_pyc_file_from_path(path):
     """Given a python source path, return the so-called
     "sourceless" .pyc or .pyo path.
@@ -238,6 +267,7 @@ def simple_pyc_file_from_path(path):
     else:
         return path + "c"  # e.g. .pyc
 
+
 def pyc_file_from_path(path):
     """Given a python source path, locate the .pyc.
 
@@ -253,11 +283,36 @@ def pyc_file_from_path(path):
     else:
         return simple_pyc_file_from_path(path)
 
+
 def rev_id():
     val = int(uuid.uuid4()) % 100000000000000
     return hex(val)[2:-1]
 
+
+def to_tuple(x, default=None):
+    if x is None:
+        return default
+    elif isinstance(x, string_types):
+        return (x, )
+    elif isinstance(x, collections.Iterable):
+        return tuple(x)
+    else:
+        raise ValueError("Don't know how to turn %r into a tuple" % x)
+
+
+def format_as_comma(value):
+    if value is None:
+        return ""
+    elif isinstance(value, string_types):
+        return value
+    elif isinstance(value, collections.Iterable):
+        return ", ".join(value)
+    else:
+        raise ValueError("Don't know how to comma-format %r" % value)
+
+
 class memoized_property(object):
+
     """A read-only @property that is only evaluated once."""
 
     def __init__(self, fget, doc=None):
@@ -278,7 +333,7 @@ class immutabledict(dict):
         raise TypeError("%s object is immutable" % self.__class__.__name__)
 
     __delitem__ = __setitem__ = __setattr__ = \
-    clear = pop = popitem = setdefault = \
+        clear = pop = popitem = setdefault = \
         update = _immutable
 
     def __new__(cls, *args):
@@ -332,7 +387,7 @@ def _with_legacy_names(translations):
             return fn(*arg, **kw)
 
         code = 'lambda %(args)s: %(target)s(%(apply_kw)s)' % (
-                metadata)
+            metadata)
         decorated = eval(code, {"target": go})
         decorated.__defaults__ = getattr(fn, '__func__', fn).__defaults__
         update_wrapper(decorated, fn)
@@ -346,6 +401,3 @@ def _with_legacy_names(translations):
         return decorated
 
     return decorate
-
-
-
