@@ -91,6 +91,25 @@ class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestBase):
         else:
             eq_(diffs, [])
 
+    def test_unique_flag_nothing_changed(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('unq_idx', m1,
+              Column('id', Integer, primary_key=True),
+              Column('x', String(20)),
+              Index('x', 'x', unique=True)
+              )
+
+        Table('unq_idx', m2,
+              Column('id', Integer, primary_key=True),
+              Column('x', String(20)),
+              Index('x', 'x', unique=True)
+              )
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
     def test_index_becomes_unique(self):
         m1 = MetaData()
         m2 = MetaData()
@@ -239,6 +258,40 @@ class AutogenerateUniqueIndexTest(AutogenFixtureTest, TestBase):
         diffs = self._fixture(m1, m2)
         eq_(diffs, [])
 
+    def test_nothing_changed_unique_w_colkeys(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('nothing_changed', m1,
+              Column('x', String(20), key='nx'),
+              UniqueConstraint('nx')
+              )
+
+        Table('nothing_changed', m2,
+              Column('x', String(20), key='nx'),
+              UniqueConstraint('nx')
+              )
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
+    def test_nothing_changed_index_w_colkeys(self):
+        m1 = MetaData()
+        m2 = MetaData()
+
+        Table('nothing_changed', m1,
+              Column('x', String(20), key='nx'),
+              Index('foobar', 'nx')
+              )
+
+        Table('nothing_changed', m2,
+              Column('x', String(20), key='nx'),
+              Index('foobar', 'nx')
+              )
+
+        diffs = self._fixture(m1, m2)
+        eq_(diffs, [])
+
     def test_nothing_changed_index_named_as_column(self):
         m1 = MetaData()
         m2 = MetaData()
@@ -587,6 +640,7 @@ class MySQLUniqueIndexTest(AutogenerateUniqueIndexTest):
             assert False, "unexpected success"
 
 
+
 class NoUqReflectionIndexTest(NoUqReflection, AutogenerateUniqueIndexTest):
     reports_unique_constraints = False
     __only_on__ = 'sqlite'
