@@ -9,6 +9,7 @@ from sqlalchemy import MetaData, Column, Table, String, \
     PrimaryKeyConstraint, Index, func, text, DefaultClause
 
 from sqlalchemy.types import TIMESTAMP
+from sqlalchemy.types import UserDefinedType
 from sqlalchemy.dialects import mysql, postgresql
 from sqlalchemy.engine.default import DefaultDialect
 from sqlalchemy.sql import and_, column, literal_column, false
@@ -18,6 +19,9 @@ from alembic.testing.mock import patch
 from alembic import autogenerate, util, compat
 from alembic.testing import eq_, eq_ignore_whitespace, config
 
+from alembic.testing.fixtures import op_fixture
+from alembic import op  # noqa
+import sqlalchemy as sa  # noqa
 
 py3k = sys.version_info >= (3, )
 
@@ -220,7 +224,7 @@ unique=False, """
             "['code'], schema='CamelSchema')"
         )
 
-    def test_drop_constraint(self):
+    def test_drop_unique_constraint(self):
         """
         autogenerate.render._drop_constraint
         """
@@ -233,10 +237,10 @@ unique=False, """
         uq = UniqueConstraint(t.c.code, name='uq_test_code')
         eq_ignore_whitespace(
             autogenerate.render._drop_constraint(uq, self.autogen_context),
-            "op.drop_constraint('uq_test_code', 'test')"
+            "op.drop_constraint('uq_test_code', 'test', type_='unique')"
         )
 
-    def test_drop_constraint_schema(self):
+    def test_drop_unique_constraint_schema(self):
         """
         autogenerate.render._drop_constraint using schema
         """
@@ -250,7 +254,126 @@ unique=False, """
         uq = UniqueConstraint(t.c.code, name='uq_test_code')
         eq_ignore_whitespace(
             autogenerate.render._drop_constraint(uq, self.autogen_context),
-            "op.drop_constraint('uq_test_code', 'test', schema='CamelSchema')"
+            "op.drop_constraint('uq_test_code', 'test', "
+            "schema='CamelSchema', type_='unique')"
+        )
+
+    def test_add_fk_constraint(self):
+        m = MetaData()
+        Table('a', m, Column('id', Integer, primary_key=True))
+        b = Table('b', m, Column('a_id', Integer, ForeignKey('a.id')))
+        fk = ForeignKeyConstraint(['a_id'], ['a.id'], name='fk_a_id')
+        b.append_constraint(fk)
+        eq_ignore_whitespace(
+            autogenerate.render._add_fk_constraint(fk, self.autogen_context),
+            "op.create_foreign_key('fk_a_id', 'b', 'a', ['a_id'], ['id'])"
+        )
+
+    def test_add_fk_constraint_inline_colkeys(self):
+        m = MetaData()
+        Table('a', m, Column('id', Integer, key='aid', primary_key=True))
+        b = Table(
+            'b', m,
+            Column('a_id', Integer, ForeignKey('a.aid'), key='baid'))
+
+        py_code = autogenerate.render._add_table(b, self.autogen_context)
+
+        eq_ignore_whitespace(
+            py_code,
+            "op.create_table('b',"
+            "sa.Column('a_id', sa.Integer(), nullable=True),"
+            "sa.ForeignKeyConstraint(['a_id'], ['a.id'], ))"
+        )
+
+        context = op_fixture()
+        eval(py_code)
+        context.assert_(
+            "CREATE TABLE b (a_id INTEGER, "
+            "FOREIGN KEY(a_id) REFERENCES a (id))")
+
+    def test_add_fk_constraint_separate_colkeys(self):
+        m = MetaData()
+        Table('a', m, Column('id', Integer, key='aid', primary_key=True))
+        b = Table('b', m, Column('a_id', Integer, key='baid'))
+        fk = ForeignKeyConstraint(['baid'], ['a.aid'], name='fk_a_id')
+        b.append_constraint(fk)
+
+        py_code = autogenerate.render._add_table(b, self.autogen_context)
+
+        eq_ignore_whitespace(
+            py_code,
+            "op.create_table('b',"
+            "sa.Column('a_id', sa.Integer(), nullable=True),"
+            "sa.ForeignKeyConstraint(['a_id'], ['a.id'], name='fk_a_id'))"
+        )
+
+        context = op_fixture()
+        eval(py_code)
+        context.assert_(
+            "CREATE TABLE b (a_id INTEGER, CONSTRAINT "
+            "fk_a_id FOREIGN KEY(a_id) REFERENCES a (id))")
+
+        context = op_fixture()
+        py_code = autogenerate.render._add_fk_constraint(
+            fk, self.autogen_context)
+
+        eq_ignore_whitespace(
+            autogenerate.render._add_fk_constraint(fk, self.autogen_context),
+            "op.create_foreign_key('fk_a_id', 'b', 'a', ['a_id'], ['id'])"
+        )
+
+        eval(py_code)
+        context.assert_(
+            "ALTER TABLE b ADD CONSTRAINT fk_a_id "
+            "FOREIGN KEY(a_id) REFERENCES a (id)")
+
+    def test_add_fk_constraint_schema(self):
+        m = MetaData()
+        Table(
+            'a', m, Column('id', Integer, primary_key=True),
+            schema="CamelSchemaTwo")
+        b = Table(
+            'b', m, Column('a_id', Integer, ForeignKey('a.id')),
+            schema="CamelSchemaOne")
+        fk = ForeignKeyConstraint(
+            ["a_id"],
+            ["CamelSchemaTwo.a.id"], name='fk_a_id')
+        b.append_constraint(fk)
+        eq_ignore_whitespace(
+            autogenerate.render._add_fk_constraint(fk, self.autogen_context),
+            "op.create_foreign_key('fk_a_id', 'b', 'a', ['a_id'], ['id'],"
+            " source_schema='CamelSchemaOne', "
+            "referent_schema='CamelSchemaTwo')"
+        )
+
+    def test_drop_fk_constraint(self):
+        m = MetaData()
+        Table('a', m, Column('id', Integer, primary_key=True))
+        b = Table('b', m, Column('a_id', Integer, ForeignKey('a.id')))
+        fk = ForeignKeyConstraint(['a_id'], ['a.id'], name='fk_a_id')
+        b.append_constraint(fk)
+        eq_ignore_whitespace(
+            autogenerate.render._drop_constraint(fk, self.autogen_context),
+            "op.drop_constraint('fk_a_id', 'b', type_='foreignkey')"
+        )
+
+    def test_drop_fk_constraint_schema(self):
+        m = MetaData()
+        Table(
+            'a', m, Column('id', Integer, primary_key=True),
+            schema="CamelSchemaTwo")
+        b = Table(
+            'b', m, Column('a_id', Integer, ForeignKey('a.id')),
+            schema="CamelSchemaOne")
+        fk = ForeignKeyConstraint(
+            ["a_id"],
+            ["CamelSchemaTwo.a.id"], name='fk_a_id')
+        b.append_constraint(fk)
+
+        eq_ignore_whitespace(
+            autogenerate.render._drop_constraint(fk, self.autogen_context),
+            "op.drop_constraint('fk_a_id', 'b', schema='CamelSchemaOne', "
+            "type_='foreignkey')"
         )
 
     def test_render_table_upgrade(self):
@@ -816,6 +939,15 @@ unique=False, """
             "sa.Enum('one', 'two', 'three')"
         )
 
+    @config.requirements.fail_before_sqla_099
+    def test_render_non_native_enum(self):
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(
+                Enum("one", "two", "three", native_enum=False),
+                self.autogen_context),
+            "sa.Enum('one', 'two', 'three', native_enum=False)"
+        )
+
     def test_repr_plain_sqla_type(self):
         type_ = Integer()
         autogen_context = {
@@ -831,9 +963,29 @@ unique=False, """
             "sa.Integer()"
         )
 
-    def test_repr_user_type_user_prefix_None(self):
-        from sqlalchemy.types import UserDefinedType
+    def test_repr_custom_type_w_sqla_prefix(self):
+        autogen_context = {
+            'opts': {
+                'sqlalchemy_module_prefix': 'sa.',
+                'alembic_module_prefix': 'op.',
+                'user_module_prefix': None
+            },
+            'dialect': mysql.dialect()
+        }
 
+        class MyType(UserDefinedType):
+            pass
+
+        MyType.__module__ = "sqlalchemy_util.types"
+
+        type_ = MyType()
+
+        eq_ignore_whitespace(
+            autogenerate.render._repr_type(type_, autogen_context),
+            "sqlalchemy_util.types.MyType()"
+        )
+
+    def test_repr_user_type_user_prefix_None(self):
         class MyType(UserDefinedType):
 
             def get_col_spec(self):
