@@ -1,23 +1,27 @@
-from unittest import TestCase
 
 from sqlalchemy import DateTime, MetaData, Table, Column, text, Integer, \
-    String, Interval
+    String, Interval, Sequence, Numeric, BigInteger, Float, Numeric
 from sqlalchemy.dialects.postgresql import ARRAY
-from sqlalchemy.schema import DefaultClause
 from sqlalchemy.engine.reflection import Inspector
 from alembic.operations import Operations
 from sqlalchemy.sql import table, column
-from alembic.autogenerate.compare import _compare_server_default
+from alembic.autogenerate.compare import \
+    _compare_server_default, _compare_tables, _render_server_default_for_compare
 
 from alembic import command, util
 from alembic.migration import MigrationContext
 from alembic.script import ScriptDirectory
-from . import db_for_dialect, eq_, staging_env, \
-    clear_staging_env, _no_sql_testing_config,\
-    capture_context_buffer, requires_09, write_script
+
+from alembic.testing import eq_, provide_metadata
+from alembic.testing.env import staging_env, clear_staging_env, \
+    _no_sql_testing_config, write_script
+from alembic.testing.fixtures import capture_context_buffer
+from alembic.testing.fixtures import TestBase
+
+from alembic.testing import config
 
 
-class PGOfflineEnumTest(TestCase):
+class PGOfflineEnumTest(TestBase):
 
     def setUp(self):
         staging_env()
@@ -71,7 +75,7 @@ def downgrade():
 
 """ % self.rid)
 
-    @requires_09
+    @config.requirements.sqlalchemy_09
     def test_offline_inline_enum_create(self):
         self._inline_enum_script()
         with capture_context_buffer() as buf:
@@ -88,7 +92,7 @@ def downgrade():
         # no drop since we didn't emit events
         assert "DROP TYPE pgenum" not in buf.getvalue()
 
-    @requires_09
+    @config.requirements.sqlalchemy_09
     def test_offline_distinct_enum_create(self):
         self._distinct_enum_script()
         with capture_context_buffer() as buf:
@@ -105,11 +109,12 @@ def downgrade():
         assert "DROP TYPE pgenum" in buf.getvalue()
 
 
-class PostgresqlInlineLiteralTest(TestCase):
+class PostgresqlInlineLiteralTest(TestBase):
+    __only_on__ = 'postgresql'
 
     @classmethod
     def setup_class(cls):
-        cls.bind = db_for_dialect("postgresql")
+        cls.bind = config.db
         cls.bind.execute("""
             create table tab (
                 col varchar(50)
@@ -150,11 +155,12 @@ class PostgresqlInlineLiteralTest(TestCase):
         )
 
 
-class PostgresqlDefaultCompareTest(TestCase):
+class PostgresqlDefaultCompareTest(TestBase):
+    __only_on__ = 'postgresql'
 
     @classmethod
     def setup_class(cls):
-        cls.bind = db_for_dialect("postgresql")
+        cls.bind = config.db
         staging_env()
         context = MigrationContext.configure(
             connection=cls.bind.connect(),
@@ -168,7 +174,13 @@ class PostgresqlDefaultCompareTest(TestCase):
             'imports': set(),
             'connection': connection,
             'dialect': connection.dialect,
-            'context': context
+            'context': context,
+            'opts': {
+                'compare_type': True,
+                'compare_server_default': True,
+                'alembic_module_prefix': 'op.',
+                'sqlalchemy_module_prefix': 'sa.',
+            }
         }
 
     @classmethod
@@ -181,8 +193,11 @@ class PostgresqlDefaultCompareTest(TestCase):
     def tearDown(self):
         self.metadata.drop_all()
 
-    def _compare_default_roundtrip(self, type_, orig_default, alternate=None):
-        diff_expected = alternate is not None
+    def _compare_default_roundtrip(
+            self, type_, orig_default, alternate=None, diff_expected=None):
+        diff_expected = diff_expected \
+            if diff_expected is not None \
+            else alternate is not None
         if alternate is None:
             alternate = orig_default
 
@@ -262,6 +277,67 @@ class PostgresqlDefaultCompareTest(TestCase):
             text("5"), "7"
         )
 
+    def test_compare_float_str(self):
+        self._compare_default_roundtrip(
+            Float(),
+            "5.2",
+        )
+
+    def test_compare_float_text(self):
+        self._compare_default_roundtrip(
+            Float(),
+            text("5.2"),
+        )
+
+    def test_compare_float_no_diff1(self):
+        self._compare_default_roundtrip(
+            Float(),
+            text("5.2"), "5.2",
+            diff_expected=False
+        )
+
+    def test_compare_float_no_diff2(self):
+        self._compare_default_roundtrip(
+            Float(),
+            "5.2", text("5.2"),
+            diff_expected=False
+        )
+
+    def test_compare_float_no_diff3(self):
+        self._compare_default_roundtrip(
+            Float(),
+            text("5"), text("5.0"),
+            diff_expected=False
+        )
+
+    def test_compare_float_no_diff4(self):
+        self._compare_default_roundtrip(
+            Float(),
+            "5", "5.0",
+            diff_expected=False
+        )
+
+    def test_compare_float_no_diff5(self):
+        self._compare_default_roundtrip(
+            Float(),
+            text("5"), "5.0",
+            diff_expected=False
+        )
+
+    def test_compare_float_no_diff6(self):
+        self._compare_default_roundtrip(
+            Float(),
+            "5", text("5.0"),
+            diff_expected=False
+        )
+
+    def test_compare_numeric_no_diff(self):
+        self._compare_default_roundtrip(
+            Numeric(),
+            text("5"), "5.0",
+            diff_expected=False
+        )
+
     def test_compare_character_str(self):
         self._compare_default_roundtrip(
             String(),
@@ -299,3 +375,104 @@ class PostgresqlDefaultCompareTest(TestCase):
         assert not self._compare_default(
             t1, t2, t2.c.id, ""
         )
+
+
+class PostgresqlDetectSerialTest(TestBase):
+    __only_on__ = 'postgresql'
+
+    @classmethod
+    def setup_class(cls):
+        cls.bind = config.db
+        cls.conn = cls.bind.connect()
+        staging_env()
+        context = MigrationContext.configure(
+            connection=cls.conn,
+            opts={
+                'compare_type': True,
+                'compare_server_default': True
+            }
+        )
+        connection = context.bind
+        cls.autogen_context = {
+            'imports': set(),
+            'connection': connection,
+            'dialect': connection.dialect,
+            'context': context,
+            'opts': {
+                'compare_type': True,
+                'compare_server_default': True,
+                'alembic_module_prefix': 'op.',
+                'sqlalchemy_module_prefix': 'sa.',
+            }
+        }
+
+    @classmethod
+    def teardown_class(cls):
+        cls.conn.close()
+        clear_staging_env()
+
+    @provide_metadata
+    def _expect_default(self, c_expected, col, seq=None):
+        Table('t', self.metadata, col)
+
+        if seq:
+            seq._set_metadata(self.metadata)
+        self.metadata.create_all(config.db)
+
+        insp = Inspector.from_engine(config.db)
+        diffs = []
+        _compare_tables(
+            set([(None, 't')]), set([]),
+            [],
+            insp, self.metadata, diffs, self.autogen_context)
+        tab = diffs[0][1]
+        eq_(_render_server_default_for_compare(
+            tab.c.x.server_default, tab.c.x, self.autogen_context),
+            c_expected)
+
+        insp = Inspector.from_engine(config.db)
+        diffs = []
+        m2 = MetaData()
+        Table('t', m2, Column('x', BigInteger()))
+        _compare_tables(
+            set([(None, 't')]), set([(None, 't')]),
+            [],
+            insp, m2, diffs, self.autogen_context)
+        server_default = diffs[0][0][4]['existing_server_default']
+        eq_(_render_server_default_for_compare(
+            server_default, tab.c.x, self.autogen_context),
+            c_expected)
+
+    def test_serial(self):
+        self._expect_default(
+            None,
+            Column('x', Integer, primary_key=True)
+        )
+
+    def test_separate_seq(self):
+        seq = Sequence("x_id_seq")
+        self._expect_default(
+            "nextval('x_id_seq'::regclass)",
+            Column(
+                'x', Integer,
+                server_default=seq.next_value(), primary_key=True),
+            seq
+        )
+
+    def test_numeric(self):
+        seq = Sequence("x_id_seq")
+        self._expect_default(
+            "nextval('x_id_seq'::regclass)",
+            Column(
+                'x', Numeric(8, 2), server_default=seq.next_value(),
+                primary_key=True),
+            seq
+        )
+
+    def test_no_default(self):
+        self._expect_default(
+            None,
+            Column('x', Integer, autoincrement=False, primary_key=True)
+        )
+
+
